use crate::renderer::context_element::IntoContextElement;
use std::borrow::Borrow;

pub trait BreadcrumbTree {
    fn get_ice(&self) -> &dyn IntoContextElement;

    fn get_parent(&self) -> Option<&dyn BreadcrumbTree>;
}

pub struct BreadcrumbTreeNode<'a> {
    parent: Option<&'a dyn BreadcrumbTree>,
    element: BreadcrumbTreeNodeElement<'a>,
}

pub enum BreadcrumbTreeNodeElement<'a> {
    Owned(Box<dyn IntoContextElement>),
    Borrowed(&'a dyn IntoContextElement),
}

impl<'a> BreadcrumbTreeNode<'a> {
    pub fn new(
        parent: Option<&'a dyn BreadcrumbTree>,
        element: BreadcrumbTreeNodeElement<'a>,
    ) -> Self {
        BreadcrumbTreeNode {
            parent: parent,
            element: element,
        }
    }

    pub fn ice_iter(&'a self) -> impl Iterator<Item = &dyn IntoContextElement> {
        self.breadcrumb_iter().map(|b| b.get_ice())
    }

    pub fn breadcrumb_iter(&'a self) -> BreadcrumbTreeIterator<'a> {
        BreadcrumbTreeIterator(Some(self))
    }
}

impl<'a> Borrow<dyn IntoContextElement + 'a> for BreadcrumbTreeNodeElement<'a> {
    fn borrow(&self) -> &(dyn IntoContextElement + 'a) {
        match self {
            BreadcrumbTreeNodeElement::Owned(ice) => ice.as_ref(),
            BreadcrumbTreeNodeElement::Borrowed(ice) => *ice,
        }
    }
}

impl<'a> BreadcrumbTree for BreadcrumbTreeNode<'a> {
    fn get_ice(&self) -> &dyn IntoContextElement {
        self.element.borrow()
    }

    fn get_parent(&self) -> Option<&dyn BreadcrumbTree> {
        self.parent
    }
}

impl<'a> IntoIterator for &'a BreadcrumbTreeNode<'a> {
    type Item = &'a dyn BreadcrumbTree;
    type IntoIter = BreadcrumbTreeIterator<'a>;

    fn into_iter(self) -> BreadcrumbTreeIterator<'a> {
        self.breadcrumb_iter()
    }
}

pub struct BreadcrumbTreeIterator<'a>(Option<&'a dyn BreadcrumbTree>);

impl<'a> Iterator for BreadcrumbTreeIterator<'a> {
    type Item = &'a dyn BreadcrumbTree;

    fn next(&mut self) -> Option<Self::Item> {
        let ret = self.0;
        self.0 = self.0.map(|node| node.get_parent()).flatten();
        ret
    }
}
